@@ -38,10 +38,11 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)cmd3.c	2.78 (gritter) 1/7/05";
+static char sccsid[] = "@(#)cmd3.c	2.81 (gritter) 4/13/05";
 #endif
 #endif /* not lint */
 
+#include <math.h>
 #include <float.h>
 #include "rcv.h"
 #include "extern.h"
@@ -364,7 +365,7 @@ respond_internal(int *msgvec, int recipient_record)
 	}
 	make_ref_and_cs(mp, &head);
 	if (mail1(&head, 1, mp, NULL, recipient_record, 0, 0) == OKAY &&
-			value("markanswered"))
+			value("markanswered") && (mp->m_flag & MANSWERED) == 0)
 		mp->m_flag |= MANSWER|MANSWERED;
 	return(0);
 }
@@ -516,6 +517,9 @@ preserve(void *v)
 		mp->m_flag |= MPRESERVE;
 		mp->m_flag &= ~MBOX;
 		setdot(mp);
+		/*
+		 * This is now Austin Group Request XCU #20.
+		 */
 		did_print_dot = 1;
 	}
 	return(0);
@@ -536,6 +540,10 @@ unread(void *v)
 		dot->m_flag |= MSTATUS;
 		if (mb.mb_type == MB_IMAP || mb.mb_type == MB_CACHE)
 			imap_unread(&message[*ip-1], *ip);
+		/*
+		 * The "unread" command is not part of POSIX mailx.
+		 */
+		did_print_dot = 1;
 	}
 	return(0);
 }
@@ -969,7 +977,7 @@ Respond_internal(int *msgvec, int recipient_record)
 	head.h_subject = reedit(head.h_subject);
 	make_ref_and_cs(mp, &head);
 	if (mail1(&head, 1, mp, NULL, recipient_record, 0, 0) == OKAY &&
-			value("markanswered"))
+			value("markanswered") && (mp->m_flag & MANSWERED) == 0)
 		mp->m_flag |= MANSWER|MANSWERED;
 	return 0;
 }
@@ -1465,10 +1473,11 @@ cundraft(void *v)
 static float 
 huge(void)
 {
+#if defined (_CRAY)
 	/*
 	 * This is not perfect, but correct for machines with a 32-bit
-	 * IEEE float, and does at least not produce SIGFPE on the Cray
-	 * Y-MP.
+	 * IEEE float and a 32-bit unsigned long, and does at least not
+	 * produce SIGFPE on the Cray Y-MP.
 	 */
 	union {
 		float	f;
@@ -1477,6 +1486,15 @@ huge(void)
 
 	u.l = 0xff800000; /* -inf */
 	return u.f;
+#elif defined (INFINITY)
+	return -INFINITY;
+#elif defined (HUGE_VALF)
+	return -HUGE_VALF;
+#elif defined (FLT_MAX)
+	return -FLT_MAX;
+#else
+	return -1e10;
+#endif
 }
 
 int 
