@@ -35,7 +35,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	Sccsid @(#)extern.h	2.151 (gritter) 12/25/04
+ *	Sccsid @(#)extern.h	2.155 (gritter) 7/13/05
  */
 
 /* aux.c */
@@ -88,6 +88,7 @@ void cwrelse(struct cw *cw);
 void makeprint(struct str *in, struct str *out);
 char *prstr(const char *s);
 int prout(const char *s, size_t sz, FILE *fp);
+int putuc(int u, int c, FILE *fp);
 int asccasecmp(const char *s1, const char *s2);
 int ascncasecmp(const char *s1, const char *s2, size_t sz);
 char *asccasestr(const char *haystack, const char *xneedle);
@@ -281,6 +282,8 @@ time_t rfctime(char *date);
 time_t combinetime(int year, int month, int day,
 		int hour, int minute, int second);
 void substdate(struct message *m);
+int check_from_and_sender(struct name *fromfield, struct name *senderfield);
+char *getsender(struct message *m);
 /* imap.c */
 enum okay imap_noop(void);
 enum okay imap_select(struct mailbox *mp, off_t *size, int *count,
@@ -408,7 +411,7 @@ enum okay ssl_open(const char *server, struct sock *sp, const char *uhp);
 void ssl_gen_err(const char *fmt, ...);
 #endif	/* USE_OPENSSL */
 int cverify(void *vp);
-FILE *smime_sign(FILE *ip);
+FILE *smime_sign(FILE *ip, struct header *);
 FILE *smime_encrypt(FILE *ip, const char *certfile, const char *to);
 struct message *smime_decrypt(struct message *m, const char *to,
 		const char *cc, int signcall);
@@ -465,8 +468,9 @@ int puthead(struct header *hp, FILE *fo, enum gfield w,
 int resend_msg(struct message *mp, struct name *to, int add_resent);
 /* smtp.c */
 char *nodename(int mayoverride);
-char *myaddr(void);
-int smtp_mta(char *server, struct name *to, FILE *fi);
+char *myaddrs(struct header *hp);
+char *myorigin(struct header *hp);
+int smtp_mta(char *server, struct name *to, FILE *fi, struct header *hp);
 /* ssl.c */
 void ssl_set_vrfy_level(const char *uhp);
 enum okay ssl_vrfy_decide(void);
