@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,13 +38,16 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)lex.c	2.18 (gritter) 12/9/03";
+static char sccsid[] = "@(#)lex.c	2.82 (gritter) 7/5/05";
 #endif
 #endif /* not lint */
 
 #include "rcv.h"
-#include <errno.h>
 #include "extern.h"
+#include <errno.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 /*
  * Mail -- a mail program
@@ -53,12 +56,11 @@ static char sccsid[] = "@(#)lex.c	2.18 (gritter) 12/9/03";
  */
 
 static char	*prompt;
+static sighandler_type	oldpipe;
 
-static const struct cmd	*lex __P((char []));
-static int	is_prefix __P((char *, char *));
-static void	intr __P((int));
-static void	stop __P((int));
-static void	hangup __P((int));
+static const struct cmd *lex(char *Word);
+static void stop(int s);
+static void hangup(int s);
 
 /*
  * Set up editing on the given file name.
@@ -69,19 +71,19 @@ static void	hangup __P((int));
  * newmail: Check for new mail in the current folder only.
  */
 int
-setfile(name, newmail)
-	char *name;
+setfile(char *name, int newmail)
 {
 	FILE *ibuf;
-	int i;
+	int i, compressed = 0;
 	struct stat stb;
 	char isedit;
 	char *who = name[1] ? name + 1 : myname;
 	static int shudclob;
 	extern int errno;
 	size_t offset;
-	int omsgcount = 0;
+	int omsgCount = 0;
 	struct shortcut *sh;
+	struct flock	flp;
 
 	isedit = *name != '%' && ((sh = get_shortcut(name)) == NULL ||
 			*sh->sh_long != '%');
@@ -91,17 +93,30 @@ setfile(name, newmail)
 	switch (which_protocol(name)) {
 	case PROTO_FILE:
 		break;
+	case PROTO_MAILDIR:
+		return maildir_setfile(name, newmail, isedit);
 	case PROTO_POP3:
 		shudclob = 1;
 		return pop3_setfile(name, newmail, isedit);
+	case PROTO_IMAP:
+		shudclob = 1;
+		if (newmail) {
+			if (mb.mb_type == MB_CACHE)
+				return 1;
+			omsgCount = msgCount;
+		}
+		return imap_setfile(name, newmail, isedit);
 	case PROTO_UNKNOWN:
 		fprintf(stderr, catgets(catd, CATSET, 217,
 				"Cannot handle protocol: %s\n"), name);
 		return -1;
 	}
-	if ((ibuf = Fopen(name, "r")) == (FILE *)NULL) {
-		if ((!isedit && errno == ENOENT) || newmail)
+	if ((ibuf = Zopen(name, "r", &compressed)) == NULL) {
+		if ((!isedit && errno == ENOENT) || newmail) {
+			if (newmail)
+				goto nonewmail;
 			goto nomail;
+		}
 		perror(name);
 		return(-1);
 	}
@@ -109,7 +124,7 @@ setfile(name, newmail)
 	if (fstat(fileno(ibuf), &stb) < 0) {
 		Fclose(ibuf);
 		if (newmail)
-			goto nomail;
+			goto nonewmail;
 		perror("fstat");
 		return (-1);
 	}
@@ -117,7 +132,7 @@ setfile(name, newmail)
 	if (S_ISDIR(stb.st_mode)) {
 		Fclose(ibuf);
 		if (newmail)
-			goto nomail;
+			goto nonewmail;
 		errno = EISDIR;
 		perror(name);
 		return (-1);
@@ -126,7 +141,7 @@ setfile(name, newmail)
 	} else {
 		Fclose(ibuf);
 		if (newmail)
-			goto nomail;
+			goto nonewmail;
 		errno = EINVAL;
 		perror(name);
 		return (-1);
@@ -143,18 +158,32 @@ setfile(name, newmail)
 	if (shudclob && !newmail)
 		quit();
 
+#ifdef	HAVE_SOCKETS
+	if (!newmail && mb.mb_sock.s_fd >= 0)
+		sclose(&mb.mb_sock);
+#endif	/* HAVE_SOCKETS */
+
 	/*
 	 * Copy the messages into /tmp
 	 * and set pointers.
 	 */
 
+	flp.l_type = F_RDLCK;
+	flp.l_start = 0;
+	flp.l_whence = SEEK_SET;
 	if (!newmail) {
 		mb.mb_type = MB_FILE;
-		mb.mb_perm = MB_DELE|MB_EDIT;
-		if ((i = open(name, O_WRONLY)) < 0)
-			mb.mb_perm = 0;
-		else
-			close(i);
+		mb.mb_perm = Rflag ? 0 : MB_DELE|MB_EDIT;
+		mb.mb_compressed = compressed;
+		if (compressed) {
+			if (compressed & 0200)
+				mb.mb_perm = 0;
+		} else {
+			if ((i = open(name, O_WRONLY)) < 0)
+				mb.mb_perm = 0;
+			else
+				close(i);
+		}
 		if (shudclob) {
 			if (mb.mb_itf) {
 				fclose(mb.mb_itf);
@@ -169,45 +198,83 @@ setfile(name, newmail)
 		edit = isedit;
 		initbox(name);
 		offset = 0;
+		flp.l_len = 0;
+		if (!edit && fcntl(fileno(ibuf), F_SETLKW, &flp) < 0) {
+			perror("Unable to lock mailbox");
+			Fclose(ibuf);
+			return -1;
+		}
 	} else /* newmail */{
 		fseek(mb.mb_otf, 0L, SEEK_END);
 		fseek(ibuf, mailsize, SEEK_SET);
 		offset = mailsize;
-		omsgcount = msgcount;
+		omsgCount = msgCount;
+		flp.l_len = offset;
+		if (!edit && fcntl(fileno(ibuf), F_SETLKW, &flp) < 0)
+			goto nonewmail;
 	}
 	mailsize = fsize(ibuf);
 	if (newmail && mailsize <= offset) {
 		relsesigs();
-		goto nomail;
+		goto nonewmail;
 	}
 	setptr(ibuf, offset);
-	setmsize(msgcount);
+	setmsize(msgCount);
+	if (newmail && mb.mb_sorted) {
+		mb.mb_threaded = 0;
+		sort((void *)-1);
+	}
 	Fclose(ibuf);
 	relsesigs();
-	sawcom = 0;
-	if ((!edit || newmail) && msgcount == 0) {
-nomail:
+	if (!newmail)
+		sawcom = 0;
+	if ((!edit || newmail) && msgCount == 0) {
+nonewmail:
 		if (!newmail) {
 			if (value("emptystart") == NULL)
-				fprintf(stderr, catgets(catd, CATSET, 88,
+nomail:				fprintf(stderr, catgets(catd, CATSET, 88,
 						"No mail for %s\n"), who);
 		}
 		return 1;
 	}
 	if (newmail) {
+		newmailinfo(omsgCount);
+	}
+	return(0);
+}
+
+
+int 
+newmailinfo(int omsgCount)
+{
+	int	mdot;
+	int	i;
+
+	for (i = 0; i < omsgCount; i++)
+		message[i].m_flag &= ~MNEWEST;
+	if (msgCount > omsgCount) {
+		for (i = omsgCount; i < msgCount; i++)
+			message[i].m_flag |= MNEWEST;
 		printf(catgets(catd, CATSET, 158, "New mail has arrived.\n"));
-		if (msgcount - omsgcount == 1)
+		if (msgCount - omsgCount == 1)
 			printf(catgets(catd, CATSET, 214,
 				"Loaded 1 new message\n"));
 		else
 			printf(catgets(catd, CATSET, 215,
 				"Loaded %d new messages\n"),
-				msgcount - omsgcount);
-		if (value("header"))
-			while (++omsgcount <= msgcount)
-				printhead(omsgcount, stdout);
+				msgCount - omsgCount);
+	} else
+		printf("Loaded %d messages\n", msgCount);
+	callhook(mailname, 1);
+	mdot = getmdot(1);
+	if (value("header")) {
+		if (mb.mb_type == MB_IMAP)
+			imap_getheaders(omsgCount+1, msgCount);
+		while (++omsgCount <= msgCount)
+			if (visible(&message[omsgCount-1]))
+				printhead(omsgCount, stdout, 0);
 	}
-	return(0);
+	return mdot;
 }
 
 static int	*msgvec;
@@ -217,47 +284,61 @@ static int	reset_on_stop;			/* do a reset() if stopped */
  * Interpret user commands one by one.  If standard input is not a tty,
  * print no prompt.
  */
-void
-commands()
+void 
+commands(void)
 {
 	int eofloop = 0;
-	int n;
-	char *linebuf = NULL;
+	int n, x;
+	char *linebuf = NULL, *av, *nv;
 	size_t linesize = 0;
-#ifdef __GNUC__
-	/* Avoid longjmp clobbering */
-	(void) &eofloop;
-#endif
 
+	(void)&eofloop;
 	if (!sourcing) {
 		if (safe_signal(SIGINT, SIG_IGN) != SIG_IGN)
-			safe_signal(SIGINT, intr);
+			safe_signal(SIGINT, onintr);
 		if (safe_signal(SIGHUP, SIG_IGN) != SIG_IGN)
 			safe_signal(SIGHUP, hangup);
 		safe_signal(SIGTSTP, stop);
 		safe_signal(SIGTTOU, stop);
 		safe_signal(SIGTTIN, stop);
 	}
+	oldpipe = safe_signal(SIGPIPE, SIG_IGN);
+	safe_signal(SIGPIPE, oldpipe);
 	setexit();
 	for (;;) {
+		interrupts = 0;
+		handlerstacktop = NULL;
 		/*
 		 * Print the prompt, if needed.  Clear out
 		 * string space, and flush the output.
 		 */
 		if (!sourcing && value("interactive") != NULL) {
-			if (is_a_tty[0] && (value("autoinc")
-						|| value("newmail"))) {
+			av = (av = value("autoinc")) ? savestr(av) : NULL;
+			nv = (nv = value("newmail")) ? savestr(nv) : NULL;
+			if (is_a_tty[0] && (av != NULL || nv != NULL ||
+					mb.mb_type == MB_IMAP)) {
 				struct stat st;
 
-				if (mb.mb_type == MB_FILE &&
+				n = (av && strcmp(av, "noimap") &&
+						strcmp(av, "nopoll")) |
+					(nv && strcmp(nv, "noimap") &&
+					 	strcmp(nv, "nopoll"));
+				x = !(av || nv);
+				if ((mb.mb_type == MB_FILE &&
 						stat(mailname, &st) == 0 &&
-						st.st_size > mailsize) {
+						st.st_size > mailsize) ||
+						(mb.mb_type == MB_IMAP &&
+						imap_newmail(n) > x) ||
+						(mb.mb_type == MB_MAILDIR &&
+						n != 0)) {
 					int odot = dot - &message[0];
 					int odid = did_print_dot;
 
 					setfile(mailname, 1);
-					dot = &message[odot];
-					did_print_dot = odid;
+					if (mb.mb_type != MB_IMAP) {
+						dot = &message[odot];
+						did_print_dot = odid;
+					}
 				}
 			}
 			reset_on_stop = 1;
@@ -299,6 +380,7 @@ commands()
 			break;
 		}
 		eofloop = 0;
+		inhook = 0;
 		if (execute(linebuf, 0, n))
 			break;
 	}
@@ -314,10 +396,7 @@ commands()
  * Contxt is non-zero if called while composing mail.
  */
 int
-execute(linebuf, contxt, linesize)
-	char linebuf[];
-	int contxt;
-	size_t linesize;
+execute(char *linebuf, int contxt, size_t linesize)
 {
 	char *word;
 	char *arglist[MAXARGC];
@@ -353,7 +432,8 @@ execute(linebuf, contxt, linesize)
 	}
 	cp2 = word;
 	if (*cp != '|') {
-		while (*cp && strchr(" \t0123456789$^.:/-+*'\",;", *cp) == NULL)
+		while (*cp && strchr(" \t0123456789$^.:/-+*'\",;(`", *cp)
+				== NULL)
 			*cp2++ = *cp++;
 	} else
 		*cp2++ = *cp++;
@@ -372,7 +452,7 @@ execute(linebuf, contxt, linesize)
 		return(0);
 	}
 	com = lex(word);
-	if (com == NONE) {
+	if (com == NULL) {
 		printf(catgets(catd, CATSET, 91,
 				"Unknown command: \"%s\"\n"), word);
 		goto out;
@@ -447,7 +527,8 @@ execute(linebuf, contxt, linesize)
 				msgvec[1] = 0;
 		}
 		if (*msgvec == 0) {
-			printf(catgets(catd, CATSET, 97,
+			if (!inhook)
+				printf(catgets(catd, CATSET, 97,
 					"No applicable messages\n"));
 			break;
 		}
@@ -534,7 +615,7 @@ out:
 	if (com == (struct cmd *)NULL)
 		return(0);
 	if (value("autoprint") != NULL && com->c_argtype & P)
-		if ((dot->m_flag & MDELETED) == 0) {
+		if (visible(dot)) {
 			muvec[0] = dot - &message[0] + 1;
 			muvec[1] = 0;
 			type(muvec);
@@ -548,13 +629,12 @@ out:
  * Set the size of the message vector used to construct argument
  * lists to message list functions.
  */
-void
-setmsize(sz)
-	int sz;
+void 
+setmsize(int sz)
 {
 
 	if (msgvec != 0)
-		free((char *) msgvec);
+		free(msgvec);
 	msgvec = (int *)scalloc((sz + 1), sizeof *msgvec);
 }
 
@@ -564,34 +644,15 @@ setmsize(sz)
  */
 
 static const struct cmd *
-lex(word)
-	char word[];
+lex(char *Word)
 {
 	extern const struct cmd cmdtab[];
 	const struct cmd *cp;
 
 	for (cp = &cmdtab[0]; cp->c_name != NULL; cp++)
-		if (is_prefix(word, cp->c_name))
+		if (is_prefix(Word, cp->c_name))
 			return(cp);
-	return(NONE);
-}
-
-/*
- * Determine if as1 is a valid prefix of as2.
- * Return true if yep.
- */
-static int
-is_prefix(as1, as2)
-	char *as1, *as2;
-{
-	char *s1, *s2;
-
-	s1 = as1;
-	s2 = as2;
-	while (*s1++ == *s2)
-		if (*s2++ == '\0')
-			return(1);
-	return(*--s1 == '\0');
+	return(NULL);
 }
 
 /*
@@ -605,11 +666,14 @@ is_prefix(as1, as2)
 static int	inithdr;		/* am printing startup headers */
 
 /*ARGSUSED*/
-static RETSIGTYPE
-intr(s)
-	int s;
+void 
+onintr(int s)
 {
-
+	if (handlerstacktop != NULL) {
+		handlerstacktop(s);
+		return;
+	}
+	safe_signal(SIGINT, onintr);
 	noreset = 0;
 	if (!inithdr)
 		sawcom++;
@@ -623,16 +687,17 @@ intr(s)
 		close(image);
 		image = -1;
 	}
-	fprintf(stderr, catgets(catd, CATSET, 102, "Interrupt\n"));
+	if (interrupts != 1)
+		fprintf(stderr, catgets(catd, CATSET, 102, "Interrupt\n"));
+	safe_signal(SIGPIPE, oldpipe);
 	reset(0);
 }
 
 /*
  * When we wake up after ^Z, reprint the prompt.
  */
-static RETSIGTYPE
-stop(s)
-	int s;
+static void 
+stop(int s)
 {
 	sighandler_type old_action = safe_signal(s, SIG_DFL);
 	sigset_t nset;
@@ -653,9 +718,8 @@ stop(s)
  * Branch here on hangup signal and simulate "exit".
  */
 /*ARGSUSED*/
-static RETSIGTYPE
-hangup(s)
-	int s;
+static void 
+hangup(int s)
 {
 
 	/* nothing to do? */
@@ -666,8 +730,8 @@ hangup(s)
  * Announce the presence of the current Mail version,
  * give the message count, and print a header listing.
  */
-void
-announce(printheaders)
+void 
+announce(int printheaders)
 {
 	int vec[2], mdot;
 
@@ -675,7 +739,7 @@ announce(printheaders)
 	vec[0] = mdot;
 	vec[1] = 0;
 	dot = &message[mdot - 1];
-	if (printheaders && msgcount > 0 && value("header") != NULL) {
+	if (printheaders && msgCount > 0 && value("header") != NULL) {
 		inithdr++;
 		headers(vec);
 		inithdr = 0;
@@ -686,53 +750,48 @@ announce(printheaders)
  * Announce information about the file we are editing.
  * Return a likely place to set dot.
  */
-int
-newfileinfo()
+int 
+newfileinfo(void)
 {
 	struct message *mp;
-	int u, n, mdot, d, s;
+	int u, n, mdot, d, s, hidden, killed, moved;
 	char fname[PATHSIZE], zname[PATHSIZE], *ename;
 
 	if (mb.mb_type == MB_VOID)
 		return 1;
-	for (mp = &message[0]; mp < &message[msgcount]; mp++)
-		if (mp->m_flag & MNEW)
-			break;
-	if (mp >= &message[msgcount])
-		for (mp = &message[0]; mp < &message[msgcount]; mp++)
-			if ((mp->m_flag & MREAD) == 0)
-				break;
-	if (mp < &message[msgcount])
-		mdot = mp - &message[0] + 1;
-	else if (value("showlast"))
-		mdot = mp - &message[0];
-	else
-		mdot = 1;
-	s = d = 0;
-	for (mp = &message[0], n = 0, u = 0; mp < &message[msgcount]; mp++) {
+	mdot = getmdot(0);
+	s = d = hidden = killed = moved =0;
+	for (mp = &message[0], n = 0, u = 0; mp < &message[msgCount]; mp++) {
 		if (mp->m_flag & MNEW)
 			n++;
 		if ((mp->m_flag & MREAD) == 0)
 			u++;
-		if (mp->m_flag & MDELETED)
+		if ((mp->m_flag & (MDELETED|MSAVED)) == (MDELETED|MSAVED))
+			moved++;
+		if ((mp->m_flag & (MDELETED|MSAVED)) == MDELETED)
 			d++;
-		if (mp->m_flag & MSAVED)
+		if ((mp->m_flag & (MDELETED|MSAVED)) == MSAVED)
 			s++;
+		if (mp->m_flag & MHIDDEN)
+			hidden++;
+		if (mp->m_flag & MKILL)
+			killed++;
 	}
 	ename = mailname;
 	if (getfold(fname, sizeof fname - 1) >= 0) {
 		strcat(fname, "/");
-		if (strncmp(fname, mailname, strlen(fname)) == 0) {
+		if (which_protocol(fname) != PROTO_IMAP &&
+				strncmp(fname, mailname, strlen(fname)) == 0) {
 			snprintf(zname, sizeof zname, "+%s",
 					mailname + strlen(fname));
 			ename = zname;
 		}
 	}
 	printf(catgets(catd, CATSET, 103, "\"%s\": "), ename);
-	if (msgcount == 1)
+	if (msgCount == 1)
 		printf(catgets(catd, CATSET, 104, "1 message"));
 	else
-		printf(catgets(catd, CATSET, 105, "%d messages"), msgcount);
+		printf(catgets(catd, CATSET, 105, "%d messages"), msgCount);
 	if (n > 0)
 		printf(catgets(catd, CATSET, 106, " %d new"), n);
 	if (u-n > 0)
@@ -741,20 +800,102 @@ newfileinfo()
 		printf(catgets(catd, CATSET, 108, " %d deleted"), d);
 	if (s > 0)
 		printf(catgets(catd, CATSET, 109, " %d saved"), s);
-	if (mb.mb_perm == 0)
+	if (moved > 0)
+		printf(catgets(catd, CATSET, 109, " %d moved"), moved);
+	if (hidden > 0)
+		printf(catgets(catd, CATSET, 109, " %d hidden"), hidden);
+	if (killed > 0)
+		printf(catgets(catd, CATSET, 109, " %d killed"), killed);
+	if (mb.mb_type == MB_CACHE)
+		printf(" [Disconnected]");
+	else if (mb.mb_perm == 0)
 		printf(catgets(catd, CATSET, 110, " [Read only]"));
 	printf("\n");
 	return(mdot);
 }
 
+int 
+getmdot(int newmail)
+{
+	struct message	*mp;
+	char	*cp;
+	int	mdot;
+	enum mflag	avoid = MHIDDEN|MKILL|MDELETED;
+
+	if (!newmail) {
+		if (value("autothread"))
+			thread(NULL);
+		else if ((cp = value("autosort")) != NULL) {
+			free(mb.mb_sorted);
+			mb.mb_sorted = sstrdup(cp);
+			sort(NULL);
+		}
+	}
+	if (mb.mb_type == MB_VOID)
+		return 1;
+	if (newmail)
+		for (mp = &message[0]; mp < &message[msgCount]; mp++)
+			if ((mp->m_flag & (MNEWEST|avoid)) == MNEWEST)
+				break;
+	if (!newmail || mp >= &message[msgCount]) {
+		for (mp = mb.mb_threaded ? threadroot : &message[0];
+				mb.mb_threaded ?
+					mp != NULL : mp < &message[msgCount];
+				mb.mb_threaded ?
+					mp = next_in_thread(mp) : mp++)
+			if ((mp->m_flag & (MNEW|avoid)) == MNEW)
+				break;
+	}
+	if (mb.mb_threaded ? mp == NULL : mp >= &message[msgCount])
+		for (mp = mb.mb_threaded ? threadroot : &message[0];
+				mb.mb_threaded ? mp != NULL:
+					mp < &message[msgCount];
+				mb.mb_threaded ? mp = next_in_thread(mp) : mp++)
+			if (mp->m_flag & MFLAGGED)
+				break;
+	if (mb.mb_threaded ? mp == NULL : mp >= &message[msgCount])
+		for (mp = mb.mb_threaded ? threadroot : &message[0];
+				mb.mb_threaded ? mp != NULL:
+					mp < &message[msgCount];
+				mb.mb_threaded ? mp = next_in_thread(mp) : mp++)
+			if ((mp->m_flag & (MREAD|avoid)) == 0)
+				break;
+	if (mb.mb_threaded ? mp != NULL : mp < &message[msgCount])
+		mdot = mp - &message[0] + 1;
+	else if (value("showlast")) {
+		if (mb.mb_threaded) {
+			for (mp = this_in_thread(threadroot, -1); mp;
+					mp = prev_in_thread(mp))
+				if ((mp->m_flag & avoid) == 0)
+					break;
+			mdot = mp ? mp - &message[0] + 1 : msgCount;
+		} else {
+			for (mp = &message[msgCount-1]; mp >= &message[0]; mp--)
+				if ((mp->m_flag & avoid) == 0)
+					break;
+			mdot = mp >= &message[0] ? mp-&message[0]+1 : msgCount;
+		}
+	} else if (mb.mb_threaded) {
+		for (mp = threadroot; mp; mp = next_in_thread(mp))
+			if ((mp->m_flag & avoid) == 0)
+				break;
+		mdot = mp ? mp - &message[0] + 1 : 1;
+	} else {
+		for (mp = &message[0]; mp < &message[msgCount]; mp++)
+			if ((mp->m_flag & avoid) == 0)
+				break;
+		mdot = mp < &message[msgCount] ? mp-&message[0]+1 : 1;
+	}
+	return mdot;
+}
+
 /*
  * Print the current version number.
  */
 
 /*ARGSUSED*/
-int
-pversion(v)
-	void *v;
+int 
+pversion(void *v)
 {
 	printf(catgets(catd, CATSET, 111, "Version %s\n"), version);
 	return(0);
@@ -763,13 +904,12 @@ pversion(v)
 /*
  * Load a file of user definitions.
  */
-void
-load(name)
-	char *name;
+void 
+load(char *name)
 {
 	FILE *in, *oldin;
 
-	if ((in = Fopen(name, "r")) == (FILE *)NULL)
+	if ((in = Fopen(name, "r")) == NULL)
 		return;
 	oldin = input;
 	input = in;
@@ -782,11 +922,11 @@ load(name)
 	Fclose(in);
 }
 
-void
-initbox(name)
-	const char *name;
+void 
+initbox(const char *name)
 {
 	char *tempMesg;
+	int dummy;
 
 	if (mb.mb_type != MB_VOID) {
 		strncpy(prevfile, mailname, PATHSIZE);
@@ -796,25 +936,30 @@ initbox(name)
 		strncpy(mailname, name, PATHSIZE);
 		mailname[PATHSIZE-1]='\0';
 	}
-	if ((mb.mb_otf = Ftemp(&tempMesg, "Rx", "w", 0600, 0)) == (FILE *)NULL) {
+	if ((mb.mb_otf = Ftemp(&tempMesg, "Rx", "w", 0600, 0)) == NULL) {
 		perror(catgets(catd, CATSET, 87,
 					"temporary mail message file"));
 		exit(1);
 	}
-	(void) fcntl(fileno(mb.mb_otf), F_SETFD, FD_CLOEXEC);
-	if ((mb.mb_itf = safe_fopen(tempMesg, "r")) == (FILE *)NULL) {
+	fcntl(fileno(mb.mb_otf), F_SETFD, FD_CLOEXEC);
+	if ((mb.mb_itf = safe_fopen(tempMesg, "r", &dummy)) == NULL) {
 		perror(tempMesg);
 		exit(1);
 	}
-	(void) fcntl(fileno(mb.mb_itf), F_SETFD, FD_CLOEXEC);
+	fcntl(fileno(mb.mb_itf), F_SETFD, FD_CLOEXEC);
 	rm(tempMesg);
 	Ftfree(&tempMesg);
-	msgcount = 0;
+	msgCount = 0;
 	if (message) {
 		free(message);
 		message = NULL;
 		msgspace = 0;
 	}
+	mb.mb_threaded = 0;
+	free(mb.mb_sorted);
+	mb.mb_sorted = NULL;
+	mb.mb_flags = MB_NOFLAGS;
 	prevdot = NULL;
 	dot = NULL;
+	did_print_dot = 0;
 }
