@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)sendout.c	2.79 (gritter) 1/10/05";
+static char sccsid[] = "@(#)sendout.c	2.86 (gritter) 7/13/05";
 #endif
 #endif /* not lint */
 
@@ -60,8 +60,7 @@ static char sccsid[] = "@(#)sendout.c	2.79 (gritter) 1/10/05";
 static char	*send_boundary;
 
 static char *getencoding(enum conversion convert);
-static struct name *fixhead(struct header *hp, struct name *tolist,
-		enum gfield addauto);
+static struct name *fixhead(struct header *hp, struct name *tolist);
 static int put_signature(FILE *fo, int convert);
 static int attach_file(struct attachment *ap, FILE *fo, int dosign);
 static int attach_message(struct attachment *ap, FILE *fo, int dosign);
@@ -70,9 +69,11 @@ static int make_multipart(struct header *hp, int convert, FILE *fi, FILE *fo,
 static FILE *infix(struct header *hp, FILE *fi, int dosign);
 static int savemail(char *name, FILE *fi);
 static int sendmail_internal(void *v, int recipient_record);
-static enum okay transfer(struct name *to, struct name *mailargs, FILE *input);
-static enum okay start_mta(struct name *to, struct name *mailargs, FILE *input);
-static void message_id(FILE *fo);
+static enum okay transfer(struct name *to, struct name *mailargs, FILE *input,
+		struct header *hp);
+static enum okay start_mta(struct name *to, struct name *mailargs, FILE *input,
+		struct header *hp);
+static void message_id(FILE *fo, struct header *hp);
 static int fmt(char *str, struct name *np, FILE *fo, int comma,
 		int dropinvalid, int domime);
 static int infix_resend(FILE *fi, FILE *fo, struct message *mp,
@@ -120,15 +121,13 @@ getencoding(enum conversion convert)
  * the distribution list into the appropriate fields.
  */
 static struct name *
-fixhead(struct header *hp, struct name *tolist, enum gfield addauto)
+fixhead(struct header *hp, struct name *tolist)
 {
 	struct name *np;
-	char	*cp;
 
 	hp->h_to = NULL;
 	hp->h_cc = NULL;
 	hp->h_bcc = NULL;
-	hp->h_replyto = NULL;
 	for (np = tolist; np != NULL; np = np->n_flink)
 		if ((np->n_type & GMASK) == GTO)
 			hp->h_to =
@@ -142,24 +141,6 @@ fixhead(struct header *hp, struct name *tolist, enum gfield addauto)
 			hp->h_bcc =
 				cat(hp->h_bcc, nalloc(np->n_fullname,
 							np->n_type|GFULL));
-		else if ((np->n_type & GMASK) == GREPLYTO)
-			hp->h_replyto =
-				cat(hp->h_replyto, nalloc(np->n_fullname,
-							np->n_type|GFULL));
-	if (addauto&GCC && (cp = value("autocc")) != NULL && *cp) {
-		np = checkaddrs(sextract(cp, GCC|GFULL));
-		hp->h_cc = cat(hp->h_cc, np);
-		tolist = cat(tolist, np);
-	}
-	if (addauto&GBCC && (cp = value("autobcc")) != NULL && *cp) {
-		np = checkaddrs(sextract(cp, GBCC|GFULL));
-		hp->h_bcc = cat(hp->h_bcc, np);
-		tolist = cat(tolist, np);
-	}
-	if (addauto&GREPLYTO && (cp = value("replyto")) != NULL && *cp) {
-		np = checkaddrs(sextract(cp, GREPLYTO|GFULL));
-		hp->h_replyto = cat(hp->h_replyto, np);
-	}
 	return tolist;
 }
 
@@ -410,8 +391,7 @@ infix(struct header *hp, FILE *fi, int dosign)
 			&isclean, dosign);
 #ifdef	HAVE_ICONV
 	tcs = gettcharset();
-	if ((convhdr = need_hdrconv(hp,
-				GTO|GSUBJECT|GCC|GBCC|GREPLYTO|GIDENT)) != 0 &&
+	if ((convhdr = need_hdrconv(hp, GTO|GSUBJECT|GCC|GBCC|GIDENT)) != 0 &&
 			asccasecmp(convhdr, tcs)) {
 		if (iconvd != (iconv_t)-1)
 			iconv_close(iconvd);
@@ -429,7 +409,7 @@ infix(struct header *hp, FILE *fi, int dosign)
 #endif	/* HAVE_ICONV */
 	if (puthead(hp, nfo,
 		   GTO|GSUBJECT|GCC|GBCC|GNL|GCOMMA|GUA|GMIME
-		   |GMSGID|GIDENT|GREPLYTO|GREF|GDATE,
+		   |GMSGID|GIDENT|GREF|GDATE,
 		   SEND_MBOX, convert, contenttype, charset)) {
 		Fclose(nfo);
 		Fclose(nfi);
@@ -447,7 +427,7 @@ infix(struct header *hp, FILE *fi, int dosign)
 		iconvd = (iconv_t)-1;
 	}
 	if ((isclean & MIME_HASNUL) == 0 && isclean & MIME_HIGHBIT &&
-			asccasecmp(charset, tcs)) {
+			charset != NULL && asccasecmp(charset, tcs)) {
 		if (iconvd != (iconv_t)-1)
 			iconv_close(iconvd);
 		if ((iconvd = iconv_open_ft(charset, tcs)) == (iconv_t)-1
@@ -704,7 +684,7 @@ Sendmail(void *v)
 }
 
 static enum okay
-transfer(struct name *to, struct name *mailargs, FILE *input)
+transfer(struct name *to, struct name *mailargs, FILE *input, struct header *hp)
 {
 	char	o[LINESIZE], *cp;
 	struct name	*np, *nt;
@@ -719,7 +699,7 @@ transfer(struct name *to, struct name *mailargs, FILE *input)
 			if ((ef = smime_encrypt(input, cp, np->n_name)) != 0) {
 				nt = nalloc(np->n_name,
 					np->n_type & ~(GFULL|GSKIN));
-				if (start_mta(nt, mailargs, ef) != OKAY)
+				if (start_mta(nt, mailargs, ef, hp) != OKAY)
 					ok = STOP;
 				Fclose(ef);
 			} else {
@@ -742,7 +722,7 @@ transfer(struct name *to, struct name *mailargs, FILE *input)
 	}
 	if (cnt) {
 		if (value("smime-force-encryption") ||
-				start_mta(to, mailargs, input) != OKAY)
+				start_mta(to, mailargs, input, hp) != OKAY)
 			ok = STOP;
 	}
 	return ok;
@@ -753,7 +733,8 @@ transfer(struct name *to, struct name *mailargs, FILE *input)
  * mailing to namelist and stdin redirected to input.
  */
 static enum okay
-start_mta(struct name *to, struct name *mailargs, FILE *input)
+start_mta(struct name *to, struct name *mailargs, FILE *input,
+		struct header *hp)
 {
 	char **args = NULL, **t;
 	pid_t pid;
@@ -771,8 +752,7 @@ start_mta(struct name *to, struct name *mailargs, FILE *input)
 			printf("\n");
 			return OKAY;
 		}
-	} else if (debug || value("debug"))
-		return OKAY;
+	}
 	/*
 	 * Fork, set up the temporary mail file as standard
 	 * input for "mail", and exec with the user list we generated
@@ -795,7 +775,7 @@ start_mta(struct name *to, struct name *mailargs, FILE *input)
 		freopen("/dev/null", "r", stdin);
 		if (smtp != NULL) {
 			prepare_child(&nset, 0, 1);
-			if (smtp_mta(smtp, to, input) == 0)
+			if (smtp_mta(smtp, to, input, hp) == 0)
 				_exit(0);
 		} else {
 			prepare_child(&nset, fileno(input), -1);
@@ -811,7 +791,8 @@ start_mta(struct name *to, struct name *mailargs, FILE *input)
 				". . . message not sent.\n"), stderr);
 		_exit(1);
 	}
-	if (value("verbose") != NULL || value("sendwait")) {
+	if (value("verbose") != NULL || value("sendwait") || debug
+			|| value("debug")) {
 		if (wait_child(pid) == 0)
 			ok = OKAY;
 		else
@@ -874,7 +855,7 @@ mail1(struct header *hp, int printheaders, struct message *quote,
 	FILE *mtf, *nmtf;
 	enum okay	ok = STOP;
 	int	dosign = -1;
-	char	*charsets, *ncs = NULL;
+	char	*charsets, *ncs = NULL, *cp;
 
 #ifdef	notdef
 	if ((hp->h_to = checkaddrs(hp->h_to)) == NULL) {
@@ -882,6 +863,11 @@ mail1(struct header *hp, int printheaders, struct message *quote,
 		return STOP;
 	}
 #endif
+	if ((cp = value("autocc")) != NULL && *cp)
+		hp->h_cc = cat(hp->h_cc, checkaddrs(sextract(cp, GCC|GFULL)));
+	if ((cp = value("autobcc")) != NULL && *cp)
+		hp->h_bcc = cat(hp->h_bcc,
+				checkaddrs(sextract(cp, GBCC|GFULL)));
 	/*
 	 * Collect user's mail from standard input.
 	 * Get the result as mtf.
@@ -932,7 +918,7 @@ mail1(struct header *hp, int printheaders, struct message *quote,
 		printf(catgets(catd, CATSET, 186, "No recipients specified\n"));
 		senderr++;
 	}
-	to = fixhead(hp, to, GCC|GBCC|GREPLYTO);
+	to = fixhead(hp, to);
 	if (hp->h_charset) {
 		wantcharset = hp->h_charset;
 		goto try;
@@ -965,7 +951,7 @@ try:	if ((nmtf = infix(hp, mtf, dosign)) == NULL) {
 	}
 	mtf = nmtf;
 	if (dosign) {
-		if ((nmtf = smime_sign(mtf)) == NULL)
+		if ((nmtf = smime_sign(mtf, hp)) == NULL)
 			goto fail;
 		Fclose(mtf);
 		mtf = nmtf;
@@ -984,7 +970,7 @@ try:	if ((nmtf = infix(hp, mtf, dosign)) == NULL) {
 	}
 	if (mightrecord(mtf, to, recipient_record) != OKAY)
 		goto out;
-	ok = transfer(to, hp->h_smopts, mtf);
+	ok = transfer(to, hp->h_smopts, mtf, hp);
 out:
 	Fclose(mtf);
 	return ok;
@@ -995,7 +981,7 @@ out:
  * Use either the host name or the from address.
  */
 static void
-message_id(FILE *fo)
+message_id(FILE *fo, struct header *hp)
 {
 	char	*cp;
 	time_t	now;
@@ -1004,7 +990,7 @@ message_id(FILE *fo)
 	if ((cp = value("hostname")) != NULL)
 		fprintf(fo, "Message-ID: <%lx.%s@%s>\n",
 				(long)now, getrandstring(24), cp);
-	else if ((cp = skin(myaddr())) != NULL && strchr(cp, '@') != NULL)
+	else if ((cp = skin(myorigin(hp))) != NULL && strchr(cp, '@') != NULL)
 		fprintf(fo, "Message-ID: <%lx.%s%%%s>\n",
 				(long)now, getrandstring(16), cp);
 }
@@ -1048,6 +1034,24 @@ mkdate(FILE *fo, const char *field)
 			tzdiff_hour, tzdiff_min);
 }
 
+static enum okay
+putname(char *line, enum gfield w, enum sendaction action, int *gotcha,
+		char *prefix, FILE *fo, struct name **xp)
+{
+	struct name	*np;
+
+	np = sextract(line, GEXTRA|GFULL);
+	if (xp)
+		*xp = np;
+	if (np == NULL)
+		return 0;
+	if (fmt(prefix, np, fo, w&(GCOMMA|GFILES), 0, action != SEND_TODISP))
+		return 1;
+	if (gotcha)
+		(*gotcha)++;
+	return 0;
+}
+
 #define	FMT_CC_AND_BCC	{ \
 				if (hp->h_cc != NULL && w & GCC) { \
 					if (fmt("Cc:", hp->h_cc, fo, \
@@ -1076,7 +1080,7 @@ puthead(struct header *hp, FILE *fo, enum gfield w,
 	int gotcha;
 	char *addr/*, *cp*/;
 	int stealthmua;
-	struct name *np;
+	struct name *np, *fromfield = NULL, *senderfield = NULL;
 
 
 	if (value("stealthmua"))
@@ -1088,28 +1092,18 @@ puthead(struct header *hp, FILE *fo, enum gfield w,
 		mkdate(fo, "Date"), gotcha++;
 	}
 	if (w & GIDENT) {
-		addr = myaddr();
-		if (addr != NULL) {
-			if (mime_name_invalid(addr, 1))
-				return 1;
-			/*
-			if ((cp = value("smtp")) == NULL
-					|| strcmp(cp, "localhost") == 0)
-				fprintf(fo, "Sender: %s\n", username());
-			*/
-			fwrite("From: ", sizeof (char), 6, fo);
-			if (mime_write(addr, sizeof *addr, strlen(addr), fo,
-					action == SEND_TODISP ?
-					CONV_NONE:CONV_TOHDR_A,
-					action == SEND_TODISP ?
-					TD_ISPR|TD_ICONV:TD_ICONV,
-					NULL, (size_t)0) == 0)
+		if (hp->h_from != NULL) {
+			if (fmt("From:", hp->h_from, fo, w&(GCOMMA|GFILES), 0,
+						action!=SEND_TODISP))
 				return 1;
 			gotcha++;
-			putc('\n', fo);
-		}
-		addr = value("ORGANIZATION");
-		if (addr != NULL) {
+			fromfield = hp->h_from;
+		} else if ((addr = myaddrs(hp)) != NULL)
+			if (putname(addr, w, action, &gotcha, "From:", fo,
+						&fromfield))
+				return 1;
+		if ((addr = hp->h_organization) != NULL ||
+				(addr = value("ORGANIZATION")) != NULL) {
 			fwrite("Organization: ", sizeof (char), 14, fo);
 			if (mime_write(addr, sizeof *addr, strlen(addr), fo,
 					action == SEND_TODISP ?
@@ -1121,12 +1115,29 @@ puthead(struct header *hp, FILE *fo, enum gfield w,
 			gotcha++;
 			putc('\n', fo);
 		}
-	}
-	if (hp->h_replyto != NULL && w & GREPLYTO) {
-		if (fmt("Reply-To:", hp->h_replyto, fo, w&(GCOMMA|GFILES), 0,
-					action != SEND_TODISP))
+		if (hp->h_replyto != NULL) {
+			if (fmt("Reply-To:", hp->h_replyto, fo,
+					w&(GCOMMA|GFILES), 0,
+					action!=SEND_TODISP))
+				return 1;
+			gotcha++;
+		} else if ((addr = value("replyto")) != NULL)
+			if (putname(addr, w, action, &gotcha, "Reply-To:", fo,
+						NULL))
+				return 1;
+		if (hp->h_sender != NULL) {
+			if (fmt("Sender:", hp->h_sender, fo,
+					w&(GCOMMA|GFILES), 0,
+					action!=SEND_TODISP))
+				return 1;
+			gotcha++;
+			senderfield = hp->h_sender;
+		} else if ((addr = value("sender")) != NULL)
+			if (putname(addr, w, action, &gotcha, "Sender:", fo,
+						&senderfield))
+				return 1;
+		if (check_from_and_sender(fromfield, senderfield))
 			return 1;
-		gotcha++;
 	}
 	if (hp->h_to != NULL && w & GTO) {
 		if (fmt("To:", hp->h_to, fo, w&(GCOMMA|GFILES), 0,
@@ -1164,7 +1175,7 @@ puthead(struct header *hp, FILE *fo, enum gfield w,
 	if (value("bsdcompat") || value("bsdorder"))
 		FMT_CC_AND_BCC
 	if (w & GMSGID && stealthmua == 0)
-		message_id(fo), gotcha++;
+		message_id(fo, hp), gotcha++;
 	if (hp->h_ref != NULL && w & GREF) {
 		fmt("References:", hp->h_ref, fo, 0, 1, 0);
 		if ((np = hp->h_ref) != NULL && np->n_name) {
@@ -1259,6 +1270,7 @@ infix_resend(FILE *fi, FILE *fo, struct message *mp, struct name *to,
 	size_t count;
 	char *buf = NULL, *cp/*, *cp2*/;
 	size_t c, bufsize = 0;
+	struct name	*fromfield = NULL, *senderfield = NULL;
 
 	count = mp->m_size;
 	/*
@@ -1267,24 +1279,15 @@ infix_resend(FILE *fi, FILE *fo, struct message *mp, struct name *to,
 	if (add_resent) {
 		fputs("Resent-", fo);
 		mkdate(fo, "Date");
-		cp = myaddr();
-		if (cp != NULL) {
-			if (mime_name_invalid(cp, 1)) {
-				if (buf)
-					free(buf);
+		if ((cp = myaddrs(NULL)) != NULL) {
+			if (putname(cp, GCOMMA, SEND_MBOX, NULL,
+					"Resent-From:", fo, &fromfield))
+				return 1;
+		}
+		if ((cp = value("sender")) != NULL) {
+			if (putname(cp, GCOMMA, SEND_MBOX, NULL,
+					"Resent-Sender:", fo, &senderfield))
 				return 1;
-			}
-			fwrite("Resent-From: ", sizeof (char), 13, fo);
-			mime_write(cp, sizeof *cp, strlen(cp), fo,
-					CONV_TOHDR_A, TD_ICONV,
-					NULL, (size_t)0);
-			putc('\n', fo);
-			/*
-			if ((cp2 = value("smtp")) == NULL
-				|| strcmp(cp2, "localhost") == 0)
-				fprintf(fo, "Resent-Sender: %s\n",
-						username());
-			*/
 		}
 #ifdef	notdef
 		/*
@@ -1312,9 +1315,11 @@ infix_resend(FILE *fi, FILE *fo, struct message *mp, struct name *to,
 		}
 		if (value("stealthmua") == NULL) {
 			fputs("Resent-", fo);
-			message_id(fo);
+			message_id(fo, NULL);
 		}
 	}
+	if (check_from_and_sender(fromfield, senderfield))
+		return 1;
 	/*
 	 * Write the original headers.
 	 */
@@ -1375,7 +1380,7 @@ resend_msg(struct message *mp, struct name *to, int add_resent)
 	if ((ibuf = setinput(&mb, mp, NEED_BODY)) == NULL)
 		return 1;
 	head.h_to = to;
-	to = fixhead(&head, to, 0);
+	to = fixhead(&head, to);
 	if (infix_resend(ibuf, nfo, mp, head.h_to, add_resent) != 0) {
 		senderr++;
 		rewind(nfo);
@@ -1396,7 +1401,7 @@ resend_msg(struct message *mp, struct name *to, int add_resent)
 	if (count(to) != 0) {
 		if (value("record-resent") == NULL ||
 				mightrecord(nfi, to, 0) == OKAY)
-			ok = transfer(to, head.h_smopts, nfi);
+			ok = transfer(to, head.h_smopts, nfi, NULL);
 	} else
 		ok = OKAY;
 	Fclose(nfi);
